require("uci_iwaddon")
require("firewall")

parser = {}
local P = {}
parser = P
-- Import Section:
-- declare everything this package needs from outside
local io = io
local wwwprint = wwwprint
if wwwprint == nil then wwwprint=print end
local oldprint = oldprint
local table = table
local type = type
local string = string
local pairs = pairs
local tonumber = tonumber
local tostring = tostring
local os = os
local tr = tr
local uci = uci
local firewall = firewall

-- no more external access after this point
setfenv(1, P)
local ifwifi = uci.get_type("wireless","wifi-iface")

local olsr = uci.get_all("olsr")
local network = uci.get_all("network")
local wireless = uci.get_all("wireless")

enable    = tonumber(olsr.webadmin.enable)    or 0
userlevel = tonumber(olsr.webadmin.userlevel) or 1
reboot    = false                -- reboot device after all apply process

local output_file = "/etc/olsrd.conf"
local ctrl_file   = "/var/etc/olsrd.ctrl"
--exe_before = {} -- execute os process in this table before any process
exe_after  = {} -- execute os process after all apply process
--exe_after["/usr/lib/webif/LUA/olsr/netinit.sh"]="Networks"
exe_after["/etc/init.d/network restart"]="Networks"
exe_after["wifi"]="Wifi"
name = "OLSR"
script = "olsrd"
init_script = "/etc/init.d/olsrd"

function process()
  wwwprint(name.." Parsers...")
  process_config()
  write_init()
  write_config()
end

function write_init()
  local wififace = uci.get("olsr","webadmin","device")
  local initfile =[[#!/bin/sh /etc/rc.common
START=50

BIN=olsrd
CONF_F=/var/etc/olsrd.conf
USE_CONF_F=/etc/olsrd.conf
DEFAULT=/etc/default/olsrd
RUN_D=/var/run
PID_F=$RUN_D/$BIN.pid
CRONSET="* * * * * /usr/lib/lua/lua-xwrt/pkgs/olsr/minute.cron"

start() {
#	include /lib/network
#	scan_interfaces
#	config_load /var/state/network
	
#	config_get WAN wan ifname
#	config_get WANDEV wan device
#	config_get LAN lan ifname
  
#  WIFI="]]..wififace..[["  
  # Allow connections to olsr info port.
#  iptables        -A input_wan      -p tcp --dport 1979 -j ACCEPT
  # OLSR needs port 698 to transmit state messages. 
#  iptables -A input_rule -p udp --dport 698 -j ACCEPT
#  iptables -A input_rule -p udp --dport 53 -j ACCEPT
#  iptables -A input_rule -p tcp --dport 53 -j ACCEPT

#  iptables -A forwarding_rule -i $WAN -o $WIFI -j ACCEPT
#  iptables -A forwarding_rule -i $WIFI -o $WAN -j ACCEPT
  # For forwarding LAN & WIFI in nodes
#  iptables -A forwarding_rule -i $LAN -o $WIFI -j ACCEPT
#  iptables -A forwarding_rule -i $WIFI -o $LAN -j ACCEPT

  # For WIFI clients to connect to nodes.
#  iptables -A forwarding_rule -i $WIFI -o $WIFI -j ACCEPT

  # For connecting a wired lan client of node 1 to wired lan client of node 2
#  iptables -A forwarding_rule -i $LAN -o $LAN -j ACCEPT
  
#  iptables -A FORWARD -i br-wifi -o br-chilli -j ACCEPT
#  iptables -A FORWARD -i br-chilli -o br-wifi -j ACCEPT
  
  # Masquerade all trafic
#  iptables -t nat -A POSTROUTING -o $WIFI -j MASQUERADE
#  iptables -t nat -A POSTROUTING -o $WAN -j MASQUERADE
#  iptables -t nat -A POSTROUTING -o $LAN -j MASQUERADE
  
  /usr/lib/lua/lua-xwrt/cron_ctrl add "$CRONSET"

	$BIN -f "$USE_CONF_F" -nofork $OPTIONS &

	# write pid file (but strip out own pid ($$))
	mkdir -p $RUN_D
	PID=`pidof $BIN`
	echo ${PID%$$} > $PID_F
  echo "OLSR is running"
}

stop() {
#	include /lib/network
#	scan_interfaces
#	config_load /var/state/network
	
#	config_get WAN wan ifname
#	config_get WANDEV wan device
#	config_get LAN lan ifname
  
#  WIFI="]]..wififace..[["  

  /usr/lib/lua/lua-xwrt/cron_ctrl del "$CRONSET"
#  iptables -t nat -D POSTROUTING -o $LAN -j MASQUERADE
#  iptables -t nat -D POSTROUTING -o $WAN -j MASQUERADE
#  iptables -t nat -D POSTROUTING -o $WIFI -j MASQUERADE
#  iptables -D forwarding_rule -i $LAN -o $LAN -j ACCEPT
#  iptables -D forwarding_rule -i $WIFI -o $WIFI -j ACCEPT
#  iptables -D forwarding_rule -i $WIFI -o $LAN -j ACCEPT
#  iptables -D forwarding_rule -i $LAN -o $WIFI -j ACCEPT
#  iptables -D forwarding_rule -i $WIFI -o $WAN -j ACCEPT
#  iptables -D forwarding_rule -i $WAN -o $WIFI -j ACCEPT

#  iptables -D input_rule -p tcp --dport 53 -j ACCEPT
#  iptables -D input_rule -p udp --dport 53 -j ACCEPT
#  iptables -D input_rule -p udp --dport 698 -j ACCEPT
#  iptables -D input_wan  -p tcp --dport 1979 -j ACCEPT
	killall olsrd
}
]]
  os.execute ("echo '"..initfile.."' > /etc/init.d/olsrd")
  wwwprint("/etc/init.d/olsrd "..tr("writed OK"))
end

function process_config()
--  local olsr = uci.get_all("olsr")
  if userlevel < 2 then
    uci.set("olsr","webadmin","ipaddr","10.128."..olsr.webadmin.nodenumber..".1")
    uci.set("olsr","webadmin","netmask","255.255.0.0")
    uci.set("olsr","webadmin","device",olsr.webadmin.ifwifi)
    uci.save("olsr")
    olsr = uci.get_all("olsr")
  end
  local netnm = olsr.webadmin.netname
  if network[netnm] == nil then 
    uci.set("network",netnm,"interface")
    uci.save("network")
    network = uci.get_all("network")
  end
  uci.set("network",netnm,"proto","static")
  uci.set("network",netnm,"type","bridge")
  uci.set("network",netnm,"ipaddr",olsr.webadmin.ipaddr)
  uci.set("network",netnm,"netmask",olsr.webadmin.netmask)
  if network[netnm].ifname == nil then
    uci.set("network",netnm,"ifname",olsr.webadmin.ifwifi)
  elseif not string.gmatch(network[netnm].ifname,olsr.webadmin.ifwifi) then
    uci.set("network",netnm,"ifname", network[netnm].ifname.." "..olsr.webadmin.ifwifi)
  end
  uci.save("network")
  network = uci.get_all("network")
  if network[netnm].type ~= "bridge" then
    if network[netnm].ifname ~= nil then
      uci.set("olsr","webadmin","device",network[netnm].ifname)
    end
  else
    uci.set("olsr","webadmin","device","br-"..netnm)
  end
  uci.save("olsr")
  olsr = uci.get_all("olsr")
  local wififace = uci.get_type("wireless","wifi-iface")
  for i=1, #wififace do
    if wififace[i].device == olsr.webadmin.ifwifi then
      uci.set("wireless",wififace[1][".name"],"ssid",olsr.webadmin.ssid)
      uci.set("wireless",wififace[1][".name"],"network",netnm)
      uci.set("wireless",wififace[1][".name"],"mode","adhoc")
      uci.set("wireless",olsr.webadmin.ifwifi,"channel",olsr.webadmin.channel)
      uci.set("wireless",olsr.webadmin.ifwifi,"disabled","0")
      break
    end
  end
	firewall.set_rule{name="OLSR",src="wifi", src_port=968, dest_port=968, proto="udp", target="ACCEPT"}
	firewall.set_rule{dest="wan", proto="tcp", dest_port=1979, target="ACCEPT"}
  uci.commit("network")
  uci.commit("wireless")
--  uci.commit("olsr")
end

function write_config()  
  local sections = uci.get_all_types("olsr")
  wwwprint ("Writing "..output_file)
  local str_gral = ""
  for k,v in pairs(sections.olsr[1]) do
    if  not string.match(k,"[.]") then
      str_gral = str_gral..k.." "..v.."\n"
    end
  end
  str_gral = str_gral.."\n"
  wwwprint ("General Settings  passed OK! ")
-- hna4 --
  local str_hna4 = ""
  if sections.Hna4 ~= nil and #sections.Hna4 > 0 then
    for i = 1, #sections.Hna4 do
      if sections.Hna4[i].NetAddr ~= nil 
      and sections.Hna4[i].NetMask ~= nil then 
        str_hna4 = str_hna4.."\t"..sections.Hna4[i].NetAddr.."\t"..sections.Hna4[i].NetMask.."\n"
      end
    end
  
    if str_hna4 ~= "" then 
      str_hna4 = "Hna4\n{\n"..str_hna4 .. "}\n\n"
    end
  end
  wwwprint ("Hna4 Settings  passed OK! ")
-- hna6 --
  local str_hna6 = ""
  if sections.Hna6 ~= nil and #sections.Hna6 > 0 then
    for i = 1, #sections.Hna6 do
      if sections.Hna6[i].NetAddr ~= nil and sections.Hna6[i].NetMask ~= nil then 
        str_hna6 = str_hna6 .. "\t" .. sections.Hna6[i].NetAddr.."\t"..sections.Hna6[i].NetMask.."\n"
      end
    end
    if str_hna6 ~= "" then
      str_hna6 = "Hna6\n{\n"..str_hna6 .. "}\n"
    end
  end
  wwwprint ("Hna6 Settings  passed OK! ")

  local str_ipc = ""
  if sections.ipc ~= nil then
    str_ipc = "IpcConnect\n{\n"
    for k, v in pairs(sections.ipc[1]) do
      if  not string.match(k,"[.]") then
        str_ipc = str_ipc .. "\t" .. k .. "\t" .. v .."\n"
      end
    end
    str_ipc = str_ipc .."\n"
    if sections.ipcHost ~= nil then
      for i = 1, #sections.ipcHost do
        str_ipc = str_ipc .. "\tHost\t"..sections.ipcHost[i].Host.."\n"
      end
    end
    str_ipc = str_ipc .."\n"
    if sections.ipcNet ~= nil then
      for i = 1, #sections.ipcNet do
        str_ipc = str_ipc .. "\tNet\t"..sections.ipcNet[i].NetAddr.."\t"..sections.ipcNet[i].NetMask.."\n"
      end
    end
    str_ipc = str_ipc .."}\n\n"
  end
  wwwprint ("IpcConnect Settings  passed OK! ")

  local str_plugins = ""
  for i = 1, #sections.LoadPlugin do
    local plugin = sections.LoadPlugin[i]
    if io.exists("/usr/lib/"..plugin.Library) then
    str_plugins = str_plugins.."LoadPlugin \""..plugin.Library.."\"\n{\n"
    for k,v in pairs(plugin) do
      if  not string.match(k,"[.]")
      and k ~= "Library" then
        if k == "hosts_file" then k = "hosts-file" end
        k = string.gsub(k,"_","-")
        str_plugins = str_plugins .. "\tPlParam\t".."\""..k.."\"\t".."\""..v.."\"\n"
      end
    end
    if sections[plugin[".name"]] ~= nil then
      for j = 1, #sections[plugin[".name"]] do
        for p, n in pairs(sections[plugin[".name"]][j]) do
          if  not string.match(p,"[.]") then
            str_plugins = str_plugins.."\tPlParam\t\""..p.."\"\t\""..n.."\"\n"
          end
        end
      end
--    str_plugins = str_plugins.."\n"
    end
    str_plugins = str_plugins .. "}\n\n"
    wwwprint (plugin.Library.." Plugin Settings  passed OK! ")
    else
      str_plugins = str_plugins.."###\n### CAUTION!!! this plugins is in your configuration file,\n### but is not Installed, plugin Name : "..plugin.Library.."\n###\n\n"
      wwwprint (str_plugins)
    end
  end 

  local str_interface = ""
  for i=1, #sections.Interface do
    local interface = sections.Interface[i]
    str_interface = str_interface .. "Interface \""..interface.Interface.."\"\n{\n"
    for k, v in pairs(interface) do
      if  not string.match(k,"[.]")
      and k ~= "Interface" then
        str_interface = str_interface .. "\t"..k.."\t"..v.."\n"
      end
    end
    str_interface = str_interface .. "}\n"
  end

  str_interface = str_interface .. "\n"
  wwwprint ("Interface Settings passed OK!")

  os.execute ("echo '".."###Conf file writen by IW-OSRL package###\n\n"..str_gral..str_ipc..str_hna4..str_hna6..str_plugins..str_interface.."' > "..output_file)
  wwwprint("Config file writed ok!")
end

return parser